@@ -4,24 +4,24 @@ skip_if(!is.null(safely(seasonal::checkX13)(fail = TRUE)$error))
 skip_if(!.Platform$OS.type == "windows" && Sys.info()["sysname"] != "Linux")
 
 test_that("Bad inputs for X11 decomposition", {
-  expect_error(
+  expect_warning(
     tsibble::pedestrian %>%
       filter(Sensor == "Southern Cross Station",
              Date == as.Date("2015-01-01")) %>%
-      feasts:::X11(Count),
+      model(feasts:::X11(Count)),
     "The X11 method only supports monthly"
   )
 
-  expect_error(
+  expect_warning(
     as_tsibble(co2) %>%
-      feasts:::X11(value ~ seq_along(value)),
+      model(feasts:::X11(value ~ seq_along(value))),
     "Exogenous regressors are not supported for X11 decompositions"
   )
 })
 
 test_that("Additive X11 decomposition", {
   tsbl_co2 <- as_tsibble(co2)
-  dcmp <- tsbl_co2 %>% feasts:::X11(value)
+  dcmp <- tsbl_co2 %>% model(feasts:::X11(value)) %>% components()
   seas_dcmp <- seasonal::seas(co2, x11="", x11.mode = "add",
                               transform.function = "none")
 
@@ -45,7 +45,7 @@ test_that("Additive X11 decomposition", {
 
 test_that("Multiplicative X11 decomposition", {
   tsbl_uad <- as_tsibble(USAccDeaths)
-  dcmp <- tsbl_uad %>% feasts:::X11(value, type = "multiplicative")
+  dcmp <- tsbl_uad %>% model(feasts:::X11(value, type = "multiplicative")) %>% components()
   seas_dcmp <- seasonal::seas(USAccDeaths, x11="", x11.mode = "mult",
                               transform.function = "log")
 
